@@ -1,7 +1,6 @@
 # ***********************************************************************************
 # Author:Tony
 # Create time:2020.12.08
-# The software is build 4 SSAP poc testing.
 # ctrl+shift+'-'
 
 #>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
@@ -107,7 +106,7 @@ def cmd():
         "[3] df.drop_duplicates() | (remove duplicate rows)\n"
         "[4] df.drop(['B', 'C'], axis=1) | (delete columns)\n"
         "[5] df1 = df.loc[df['parameter name']==xxx] | (filter dataset with conditions)\n"
-        "[6] df.dropna(axis=1) | (remove nan)\n"
+        "[6] df.dropna(axis=1) | (remove nan,delete column: axis=1, delete row: axis=0)\n"
         "[7] df = df.rename(columns = {'engine serial number':'esn'}) | (change column name)\n"
         "[8] df2.sort_values(by=['xxx'],inplace=True) | (sort values)\n"
         "[9] set(df) | (remove duplicate for one parameter)\n"
@@ -1869,6 +1868,57 @@ def network02(df, entity1, entity2, relation):
 
 
 
+# bsfcmap + dutycycle
+def bsfcmap_dutycycle(df, speed_name, torque_name, bsfc_name, stepx, stepy, bubblesize, title, level):
+    from scipy.interpolate import griddata
+    import matplotlib.pyplot as plt
+    import pandas as pd
+    import numpy as np
+
+    df = df
+    x = np.array(df[speed_name])
+    y = np.array(df[torque_name])
+    z = np.array(df[bsfc_name])
+    level = level
+    title = title
+
+    xi = np.linspace(min(x), max(x), 1000)
+    yi = np.linspace(min(y), max(y), 1000)
+    X, Y = np.meshgrid(xi, yi)
+    Z = griddata((x, y), z, (X, Y), method='cubic')
+
+    # duty cycle inputs:
+    x1 = int((min(x) - 200) / 100) * 100
+    x2 = int((max(x) + 200) / 100) * 100
+    y1 = int((min(y) - 200) / 100) * 100
+    y2 = int((max(y) + 200) / 100) * 100
+
+    xedges = [*range(x1, x2, stepx)]
+    yedges = [*range(y1, y2, stepy)]
+
+    hist_temp, xedges, yedges = np.histogram2d(x, y, bins=(xedges, yedges))
+    hist = hist_temp.T
+    xpos, ypos = np.meshgrid(xedges[:-1], yedges[:-1])
+    xpos = xpos.ravel()
+    ypos = ypos.ravel()
+
+    # Visualization(duty cycle+fuel map) method: nearest / linear / cubic
+    # (fuel map)
+    plt.figure(figsize=(12, 8))
+    C = plt.contour(X, Y, Z, level, colors='black', alpha=0.5)
+    plt.contourf(X, Y, Z, level, alpha=.75, cmap='plasma')
+    plt.colorbar()
+    plt.clabel(C, inline=True, fontsize=10, colors='black')
+    plt.xlabel(speed_name)
+    plt.ylabel(torque_name)
+    plt.title(title)
+
+    # (duty cycle)
+    plt.grid(which='both', color='grey', linestyle='--', alpha=0.6)
+    plt.scatter(xpos, ypos, s=hist * bubblesize, alpha=0.3, color='white', linewidth=1, edgecolor='black')
+    plt.show()
+
+
 
 
 
@@ -1877,16 +1927,3 @@ def network02(df, entity1, entity2, relation):
 this_is_nothing = 'Below modules are not validated.---------------------👇'
 
 
-
-
-
-# Line plot (x-时间序列: 4.移动平均图 Arima  5.时间序列预测 等种种）
-# 多项式回归拟合
-# 聚类算法 DBSCAN.
-# 多元线性回归：https://cloud.tencent.com/developer/article/1059976
-# 移动平均模型（ARIMA）
-# 时间序列预测图：自相关性，偏自相关图等等
-# wordcloud plot(词云图)
-# 贝叶斯统计方法-python实现
-# python自动化爬虫
-# 递归算法
\ No newline at end of file
